# -*- coding: utf-8 -*-
"""
MojiConv.py

description: TODO

"""
import Globals
from Globals import package_home, DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
import codecs


default_from_charset	= 'euc-jp'
default_to_charset		= 'utf-8'
default_error_option	= 'replace'

valid_charsets			= ['utf-8', 'euc-jp', 'shift_jis', 'iso-2022-jp']
codec_aliases			= {
	'euc' : 'euc-jp',
	'euc-jp' : 'euc-jp',
	'sjis' : 'shift_jis',
	'shift_jis' : 'shift_jis',
	'iso-2022-jp' : 'iso-2022-jp',
	'jis' : 'iso-2022-jp',
	'utf8' : 'utf-8',
	'utf-8' : 'utf-8',
}

def convert(s, f=None, t=None):
	"""
	convert charset of string.

	usage:
		convert(string, f, t)

		string	- string to convert
		f		- 'from' charset (before convert)
		t		- 'to' charset (after convert)

	for charset, use 'utf-8', 'euc-jp', 'shift_jis', 'iso-2022-jp' etc.

	"""
	if not f: f = default_from_charset
	f = _validate_encoding(f)
	if not t: t = default_to_charset
	t = _validate_encoding(t)
	return _doConvert(s, f, t)

def _lookup_codec(encoding):
	try:
		codecs.lookup(encoding)
	except LookupError:
		raise LookupError, "codec %s not found" %(encoding)

def _validate_encoding(encoding):
	l_encoding = encoding.lower()
	try:
		l_encoding = codec_aliases[l_encoding]
	except KeyError:
		raise 'encoding %s not found' %(encoding)
	return l_encoding

def _doConvert(s, f, t):
	""" real converting job """
	return unicode(s, f, default_error_option).encode(t, default_error_option)

DEFAULT_PERM = 'Manage properties' # What permission is best?

class MojiConv(SimpleItem):
	""" MojiConv class """

	meta_type		= 'MojiConv'
	description		= 'MojiConv'
	title			= ''

	from_charset	= default_from_charset
	to_charset		= default_to_charset
	valid_charsets	= valid_charsets

	manage_options = (
		({'label':'Settings', 'action':'manage_mojiconv',},)
		+ SimpleItem.manage_options)

	manage_mojiconv = Globals.DTMLFile('dtml/manageMojiConv', globals())

	security = ClassSecurityInfo()

	security.declareProtected('View', 'index_html')
	def index_html(self):
		""" default view """
		return 'Hi. There\'s nothing to show.'

	security.declareProtected(DEFAULT_PERM, '__call__')
	def __call__(self, s, from_charset=None, to_charset=None):
		return self.convert(s, from_charset, to_charset)

	security.declareProtected(DEFAULT_PERM, 'convert')
	def convert(self, s, from_charset=None, to_charset=None):
		return convert(s, from_charset, to_charset)

	security.declareProtected(DEFAULT_PERM, 'manage_changeSettings')
	def manage_changeMojiConv(self, title=None, from_charset=None, to_charset=None, REQUEST=None):
		""" change MojiConv settings """
		if title:
			self.title = title
		if from_charset:
			self.from_charset = from_charset
		if to_charset:
			self.to_charset = to_charset
		if REQUEST is not None:
			return self.manage_mojiconv(
				self, REQUEST, manage_tabs_message='Changes saved.'
			)

InitializeClass(MojiConv)

def _codec_installed_check():
	""" check japaneseCodecs in codecs """
	for c in valid_charsets:
		_lookup_codec(c)

def manage_addMojiConv(self, new_id, title='', REQUEST=None):
	""" add MojiConv """
	_codec_installed_check()
	obj = MojiConv()
	obj.id = str(new_id)
	obj.title = str(title)
	id = self._setObject(obj.id, obj)
	obj = getattr(self, id)
	if REQUEST is not None: return self.manage_main(self, REQUEST)

manage_addMojiConvForm = DTMLFile('dtml/addMojiConvForm', globals())
